@@ -2,23 +2,23 @@ from django_tables2 import SingleTableMixin
 from django.shortcuts import redirect
 from django.http import HttpResponseRedirect
 from django.urls import reverse_lazy
+from django.db.models import Q
 from pandas import isna, DataFrame as DF, to_datetime
 import inspect
 from django.contrib import messages
-from django.db.models import F
+from django.db.models import F, Count
+from django_filters.views import FilterView
 from django_pandas.io import read_frame
 from django.contrib.auth.mixins import UserPassesTestMixin
 import plotly.express as px
 from plotly import offline
 
-
-
-class SaveFilterMixin(SingleTableMixin):
-    """ This Mixin Can be used with a FilterView SingleTable in order to save
+class SaveFilterMixinNT:
+    """ This Mixin Can be used to save
     the users filter selections after navegating away from the lookup page"""
 
     def get(self, request, *args, **kwargs):
-        view_name = self.__class__
+        view_name = self.__class__.__name__
         qstr = request.GET.urlencode()
         full_path = request.get_full_path()
         if qstr != '':
@@ -42,10 +42,14 @@ class SaveFilterMixin(SingleTableMixin):
         else:
             return kwargs
 
+class SaveFilterMixin(SingleTableMixin, SaveFilterMixinNT):
+    ''' SaveFilterMixin Classic (for use with SingleTableMixin) '''
+
 class InlineFormsetMixin:
+    ''' This mixin allows for multiple formset factories to be injected and processed in a form view '''
     factories = [] # list of dictionaries that must contain the key factory and the value of a formset factory instance, helper and herder are optional
     form_helper = None
-    template_name = 'oee/inline-formset.html'
+    template_name = 'django_aux/inline-formset.html'
     addlines_url = None # set this in the view to have the addlines btn redirect here with self.object.pk as an arg
     
     def get_context_data(self, *args, factories=None, **kwargs):
@@ -98,13 +102,41 @@ class InlineFormsetMixin:
             #user defined a redirect url for the addlines btn, default behavior sends pk as arg
             return reverse_lazy(self.addlines_url, kwargs={'pk':self.object.pk})
         elif 'addlines' in self.request.POST and not self.addlines_url:
-            print('\n\n\nI am here\n\n\n')
             #adding lines to form but no redirect url given. Use default behavior
             return self.request.path_info
         else:
             #add lines was not clicked, use default success_url
             return super().get_success_url()
 
+class DashFilterView(FilterView):
+    ''' A view for passing filter qs to a dash app '''
+    plotly_app_name = None #set this for dash functionality
+    template_name = 'django_aux/dash-filter.html'
+    values_args = []    #passing all sorts of args and kwargs for backend qs
+    values_kwargs = {}
+    annotate_kwargs = {}
+    filter_args = []
+    filter_kwargs = {}
+    to_json_kwargs = dict(date_format='iso', orient='records')
+
+    def get_context_data(self, *args, **kwargs):   
+        context = super().get_context_data(*args, **kwargs)
+        context['plotly_app_name'] = self.plotly_app_name
+        context['style_str'] = "min-width:1500px"        
+        context['initial_arguments'] = self.get_initial_arguments()
+        return context
+        
+    def get_initial_arguments(self):
+        qs = self.object_list.filter(
+            *self.filter_args, **self.filter_kwargs
+        ).annotate(
+            **self.annotate_kwargs
+        ).values(
+            *self.values_args, **self.values_kwargs
+        )
+        df = read_frame(qs).astype(str)
+        data = df.to_json(**self.to_json_kwargs)
+        return {'data':{'children':data}}
 
 class SaveFormMixin:
     """ This Mixin Can be used with any view that uses a form mixin to
@@ -169,7 +201,6 @@ class RedirectPrevMixin:
 
     def get_success_url(self):
         ''' If next was stored redirect there, otherwise return super() '''
-        print('\n\n\nI am REdirect prev\n\n\n')
         next = self.request.session.get('next')
         if next: 
             return next
@@ -206,28 +237,222 @@ class CheckGroupPermMixin(UserPassesTestMixin):
             return True
         return False
 
+
+class PlotlyMixin:
+    ''' This mixin generates a dynamic plotly plot, must be used with a filter that inherits PlotSettingsFilterMixin '''
+    plot_width = 1000
+    plot_height = None
+    max_records = 1_000_000_000
+
+
+    def check_qs_count(self):
+        """ Performs a check to see if the queryset count is greater than the max allowed records
+
+        Returns:
+            bool: Indicting whether or the qs was too large
+        """        
+        if self.object_list.count() > self.max_records:
+            emsg = f'Number of records must be < {self.max_records} to produce plot.  Please filter more'
+            messages.warning(self.request, emsg)
+            return False
+        return True   
+
+
+    @staticmethod
+    def add_values(value, vargs, vkwargs):
+        """Static method that takes a "value" determines its type and add it to vargs or vkwargs appropriately
+        Args:
+            value (Any): either a argument or keyword argument to be passed to the Queryset values method
+            vargs (list): list of arguments to be passed to Queryset values method
+            vkwargs (dict): dict of keyword arguments to be passed to Queryset values method
+
+        """        
+        if type(value) == dict:
+            vkwargs.update(value)
+        else:
+            vargs.append(value)
+
+    def get_plot_settings(self):
+        """ Method pulls users plot settings from the get request and stores them as instance attributes """ 
+        attrs = ['x','y','color','plot_type','aggregate_by','N_min','y_min','y_max']
+        for attr in attrs:
+            val = self.request.GET.get(attr)
+            if val == '': val=None
+            setattr(self,attr,val)
+        if self.N_min in [None, '']:
+            self.N_min = 0
+        self.N_min = int(self.N_min)
+        ### Get x and y verbose
+        self.yd = self.Y_CONFIG.get(self.y)
+        self.y_verbose = self.yd.get('verbose') if self.yd else ''
+        if self.y_verbose == None:
+            self.y_verbose = self.y
+        self.x_verbose = dict(self.choices.get('X_CHOICES')).get(self.x)
+        if self.x_verbose == None:
+            self.x_verbose = self.x
+
+    def get_vargs_vkwargs(self):
+        """Method estabishes and returns args and kwargs to be passed to the
+            Queryset values method
+
+        Returns:
+            tuple: Tuple containing vargs and vkwargs
+        """          
+        vargs = []
+        vkwargs = {} 
+        for label in [self.x, self.color, self.aggregate_by]:
+            if label == None:
+                continue
+            val = self.CHOICE_VALUES_MAP.get(label)
+            self.add_values(val, vargs, vkwargs) 
+        return vargs, vkwargs  
+
+    def get_akwargs(self):
+        """Method estabishes and returns the kwargs to be passed to the
+            Queryset annotate method
+
+        Returns:
+            dict: a dict containing the annotate kwargs
+        """          
+        agg_func = self.yd.get('agg_expr')
+        akwargs = {f'{self.y}':agg_func}
+        akwargs['N'] = Count('id')
+        return akwargs 
+
+    def get_grouped_qs(self):
+        """Method generates and returns the grouped/aggregated queryset to be used in the plot
+
+        Returns:
+            Queryset: The grouped queryset to be plotted
+        """        
+        qs = self.object_list
+        vargs, vkwargs = self.get_vargs_vkwargs()
+        akwargs = self.get_akwargs()
+        fkwargs = {'N__gte':self.N_min}
+        if self.y_min: fkwargs[f'{self.y}__gte'] = self.y_min
+        if self.y_max: fkwargs[f'{self.y}__lte'] = self.y_max
+        fargs = [~Q(**{f'{self.x}':None})]
+        if self.color != None: fargs.append(~Q(**{f'{self.color}':None}))
+        if self.aggregate_by != None: fargs.append(~Q(**{f'{self.aggregate_by}':None}))
+        gqs = qs.values(*vargs, **vkwargs).annotate(**akwargs).filter(*fargs, **fkwargs).order_by(self.x)
+        return gqs
+
+    def get_px_args_kwargs_obj(self):
+        """Method generates and returns the args, kwargs and plotly.express object to be used in the plot
+
+        Returns:
+            tuple: tuple containg the args, kwargs and px object
+        """        
+        args = [self.plot_df]
+        kwargs = dict(x=self.x, y=self.y, width=self.plot_width, height=self.plot_height)
+        if self.color:
+            kwargs['color'] = self.color
+        if 'bar' in self.plot_type:
+            if self.plot_type.endswith('g'):
+                kwargs['barmode'] = 'group'
+            plot_obj = px.bar
+        if self.plot_type == 'box':
+            plot_obj = px.box
+        if self.plot_type == 'line':
+            plot_obj = px.line
+        if self.plot_type == 'violin':
+            kwargs['points'] = 'all'
+            kwargs['box'] = True
+            plot_obj = px.violin
+        if self.plot_type == 'scatter':
+            plot_obj = px.scatter
+        return args, kwargs, plot_obj
+
+    def get_fig(self):
+        """Method generates and returns a plotly Figure object using 
+            class variables set in the derived class
+
+        Returns:
+            plotly.graph_objects.Figure: A plotly Figure instance
+        """    
+        self.plot_df = DF()    
+        if self.check_qs_count() == False: 
+            return
+        self.get_plot_settings()
+        if self.x == None or self.y == None:
+            return None
+        gqs = self.get_grouped_qs()
+        self.plot_df = read_frame(gqs)
+        args, kwargs, plot_obj = self.get_px_args_kwargs_obj()
+        fig = plot_obj(*args, **kwargs)
+        fig.update_layout(
+            xaxis_title=self.x_verbose, yaxis_title=self.y_verbose
+        )
+        return offline.plot(fig, auto_open=False, output_type="div")
+
+
+    def get_filterset_kwargs(self, filterset_class):
+        kwargs = super().get_filterset_kwargs(filterset_class)
+        x_choices = self.X_CHOICES
+        agg_choices = [(None,'------')]
+        color_choices = [(None,'------')]
+        if not hasattr(self, 'COLOR_CHOICES'):
+            color_choices += self.X_CHOICES
+        else:
+            color_choices = self.COLOR_CHOICES
+        if not hasattr(self, 'AGG_CHOICES'):
+            agg_choices += self.X_CHOICES   
+        else:
+            agg_choices = self.AGG_CHOICES
+
+        y_choices = []
+        for key, yd in self.Y_CONFIG.items():
+            y_choices.append((key, yd.get('verbose')))  
+
+        choices = dict(
+            X_CHOICES=x_choices,
+            Y_CHOICES=y_choices,
+            COLOR_CHOICES=color_choices,
+            PLOT_TYPE_CHOICES=(
+                ('barg', 'Bar-Grouped'), ('bars', 'Bar-Stacked'),
+                ('line', 'Line'), ('scatter', 'Scatter'),
+                ('box', 'Box'), ('violin', 'Violin'),
+            ),
+            AGG_CHOICES=agg_choices
+        )
+        self.choices = choices
+        kwargs['choices'] = choices
+        return kwargs
+
+    def get_context_data(self, **kwargs):
+        context = super().get_context_data(**kwargs)
+        context['fig'] = self.get_fig()
+        context['plot_df'] = self.plot_df
+        return context
+
+
 class SinglePlotMixin:
     ''' This mixin creates a potly figure based on a FilterView that is using
     the PlotSettingsFilterMixin '''
     plot_width = 1000
     max_records = 1_000_000_000
 
-    def get_fig(self):
-        self.plot_df = DF()
+    def check_qs_count(self):
         if self.object_list.count() > self.max_records:
             emsg = f'Number of records must be < {self.max_records} to produce plot.  Please filter more'
             messages.warning(self.request, emsg)
+            return False
+        return True       
+
+    def get_fig(self):
+        self.plot_df = DF()
+        
+        if self.check_qs_count() == False:
             return
 
         ### Get Plot Settings ###
         x = self.request.GET.get('x')
         y = self.request.GET.get('y')
-
         color = self.request.GET.get('color')
         plot_type = self.request.GET.get('plot_type')
         agg_by = self.request.GET.get('aggregate_by')
         N_min = self.request.GET.get('N_min')
-        if x == None:
+        if x == None or y == None:
             return None
         if N_min in [None, '']:
             N_min = 0
@@ -377,10 +602,8 @@ class SinglePlotMixin:
         if hasattr(self, 'AGG_CHOICES_EXTRA'):
             for tup in self.AGG_CHOICES_EXTRA:
                 choices.get('AGG_CHOICES').append(tup)
-
         self.choices = choices
         kwargs['choices'] = choices
-
         return kwargs
 
     def get_context_data(self, **kwargs):
